@@ -2,6 +2,7 @@
 Extension of the Chord class from music21.chord to support the
 Harte notation.
 """
+# pylint: disable=consider-using-dict-items
 
 from typing import List, Union
 
@@ -31,57 +32,92 @@ class Harte(Chord):
         self.chord = chord
         try:
             parsed_chord = PARSER.parse(chord)
-        except NameError:
+        except NameError as name_error:
             raise ChordException(
-                f'The input chord {chord} is not a valid Harte chord')
+                f'The input chord {chord} is not a valid Harte chord') \
+                from name_error
 
-        assert parsed_chord['root']
+        if "root" in parsed_chord:
+            # chord is not empty
+            # retrieve information from the parsed chord
+            self._root = parsed_chord['root']
+            self._shorthand = parsed_chord[
+                'shorthand'] if 'shorthand' in parsed_chord.keys() else None
+            self._degrees = parsed_chord[
+                'degrees'] if 'degrees' in parsed_chord.keys() else None
+            self._bass = parsed_chord[
+                'bass'] if 'bass' in parsed_chord.keys() else '1'
+            removed_degrees = [x for x in self._degrees if x.startswith(
+                '*')] if self._degrees else []
 
-        # retrieve information from the parsed chord
-        self._root = parsed_chord['root']
-        self._shorthand = parsed_chord[
-            'shorthand'] if 'shorthand' in parsed_chord.keys() else None
-        self._degrees = parsed_chord[
-            'degrees'] if 'degrees' in parsed_chord.keys() else None
-        self._bass = parsed_chord[
-            'bass'] if 'bass' in parsed_chord.keys() else '1'
-        self._removed_degrees = [x for x in self._degrees if x.startswith(
-            '*')] if self._degrees else []
+            # unwrap shorthand if it exists and merge with degrees
+            # if no shorthand exists, just use the degrees
+            # if no degrees exist, assume the chord is a major triad
+            if self._shorthand:
+                assert SHORTHAND_DEGREES[self._shorthand], 'The Harte ' \
+                                                           'shorthand is' \
+                                                           ' not valid. '
+                self._shorthand_degrees = SHORTHAND_DEGREES[self._shorthand]
+                self._all_degrees = self._shorthand_degrees + self._degrees if \
+                    self._degrees else self._shorthand_degrees
+            elif self._degrees:
+                self._all_degrees = self._degrees
+            else:
+                self._all_degrees = ['1', '3', '5']
 
-        # unwrap shorthand if it exists and merge with degrees
-        # if no shorthand exists, just use the degrees
-        # if no degrees exist, assume the chord is a major triad
-        if self._shorthand:
-            assert self._shorthand in SHORTHAND_DEGREES.keys(), 'The Harte ' \
-                                                                'shorthand is' \
-                                                                ' not valid. '
-            self._shorthand_degrees = SHORTHAND_DEGREES[self._shorthand]
-            self._all_degrees = self._shorthand_degrees + self._degrees if self._degrees else self._shorthand_degrees
-        elif self._degrees:
-            self._all_degrees = self._degrees
+            self._all_degrees = [x for x in self._all_degrees if
+                                 x not in removed_degrees]
+            # add root and bass note to the overall list of degrees
+            self._all_degrees.append(self._bass)
+            self._all_degrees.append('1')
+            # sort the list and remove duplicates
+            self._all_degrees = list(set(self._all_degrees))
+            self._all_degrees.sort(
+                key=lambda x: [k for k in x if k.isdigit()][0])
+
+            # convert notes and interval to m21 primitives
+            # note that when multiple flats are introduced (i.e. Cbb) music21
+            # won't be able to parse the note.
+            # this is fixed by replacing each 'b' with a '-'.
+            m21_root = Note(self._root.replace("b", "-"))
+            m21_degrees = [HarteInterval(x).transposeNote(m21_root)
+                           for x in self._all_degrees]
+            m21_bass = HarteInterval(self._bass).transposeNote(
+                m21_root)
+            # make sure that order between notes in the chord is respected
+            # by transposing pitches higher than the preceding one when needed
+            for i in range(1, len(m21_degrees)):
+                m21_degrees[i].pitch.octave = m21_degrees[
+                    i].pitch.implicitOctave
+                m21_degrees[i].pitch.transposeAboveTarget(
+                    m21_degrees[i - 1].pitch,
+                    inPlace=True)
+            m21_bass.pitch.octave = m21_bass.pitch.implicitOctave
+            if m21_bass != m21_degrees[0]:
+                m21_bass.pitch.transposeBelowTarget(m21_degrees[0].pitch,
+                                                    inPlace=True)
+
+            # initialize the parent constructor
+            super().__init__(m21_degrees, **keywords)
+            super().root(m21_root)
+            super().bass(m21_bass)
         else:
-            self._all_degrees = ['1', '3', '5']
+            # chord is empty
+            self._root = None
+            self._shorthand = None
+            self._degrees = None
+            self._bass = None
+            super().__init__(**keywords)
 
-        self._all_degrees = [x for x in self._all_degrees if
-                             x not in self._removed_degrees]
-        # add root and bass note to the overall list of degrees
-        self._all_degrees.append(self._bass)
-        self._all_degrees.append('1')
-        # sort the list and remove duplicates
-        self._all_degrees.sort(key=lambda x: [k for k in x if k.isdigit()][0])
-        self._all_degrees = list(set(self._all_degrees))
+    def __deepcopy__(self, *args, **kwargs):
+        """
+        Perform a deepcopy of this obect by creating a new identical
+        object with the input chord used for this one.
 
-        # convert notes and interval to m21 primitives
-        self._m21_root = Note(self._root)
-        self._m21_degrees = [HarteInterval(x).transposeNote(self._m21_root)
-                             for x in self._all_degrees]
-        self._m21_bass = HarteInterval(self._bass).transposeNote(
-            self._m21_root)
-
-        # initialize the parent constructor
-        super().__init__(self._m21_degrees, **keywords)
-        super().root(self._m21_root)
-        super().bass(self._m21_bass)
+        :return: A copy of the current object.
+        :rtype: Harte
+        """
+        return Harte(self.chord)
 
     def get_degrees(self) -> List[str]:
         """
@@ -171,14 +207,14 @@ class Harte(Chord):
                     clean_harte_degrees.remove('*3')
                 break
         if shorthand:
-            clean_harte_degrees = f'({",".join([x for x in clean_harte_degrees])})' if len(
+            clean_harte_degrees = f'({",".join(clean_harte_degrees)})' if len(
                 clean_harte_degrees) > 0 else ''
             if len(shorthand) > 0 or len(clean_harte_degrees) > 0:
                 separator = ':'
             return self._root + separator + shorthand + clean_harte_degrees
         return self.chord
 
-    def unwrap_shorthand(self) -> list[str] | None:
+    def unwrap_shorthand(self) -> Union[List[str], None]:
         """
         Method to retrieve the degrees of the chord in Harte notation, both
         those associated to the shorthand and those explicitly specified
@@ -188,7 +224,7 @@ class Harte(Chord):
         """
         if self._shorthand:
             return self._all_degrees
-        elif self._degrees:
+        if self._degrees:
             return self._degrees
         return None
 
@@ -209,22 +245,33 @@ class Harte(Chord):
         Method to represent the HarteChord object as a string
         :return: a string representing the HarteChord object
         """
-        return f'Harte({self._root}:{self._shorthand}({self._degrees})/{self._bass})'
+        return f'Harte({str(self)})'
 
     def __str__(self):
         """
         Method to represent the HarteChord object as a string
         :return: a string representing the HarteChord object
         """
-        return f'{self._root}:{self._shorthand}({self._degrees})/{self._bass}'
+        if self._root is not None:
+            chord_str = self._root
+            if self._shorthand is not None:
+                chord_str += ':' + self._shorthand
+            if self._degrees is not None:
+                chord_str += '(' + self._degrees + ')'
+            if self._bass != '1':
+                chord_str += '/' + self._bass
+        else:
+            chord_str = 'N'
+
+        return chord_str
 
 
 if __name__ == '__main__':
     # test utilities
-    c = Harte('C:(3,5,7,9,11)/3')
-    root = c.get_root()
-    print(c.fullName)
-    print(c.commonName)
-    print(c.inversion())
-    print(root, c.bass(), c.get_bass())
-    print(c.prettify())
+    test_chord = Harte('C:maj7(b3,5,7)')
+    root = test_chord.get_root()
+    print(test_chord.fullName)
+    print(test_chord.commonName)
+    print(test_chord.inversion())
+    print(root, test_chord.bass(), test_chord.get_bass())
+    print(test_chord.prettify())
